@@ -7,9 +7,11 @@ from pathlib import Path, PurePath
 from typing import (
     IO,
     Any,
+    ContextManager,
     Dict,
     Generator,
     Generic,
+    Iterator,
     List,
     Optional,
     Tuple,
@@ -130,6 +132,9 @@ class Bucket:
 class BucketClient:
     """Base class for a client that interacts with a bucket-based storage system."""
 
+    def recreate(self, **kwargs: Any) -> None:
+        """Recreate any underlying bucket client adapter with the given kwargs"""
+
     def open(
         self,
         path: "Pathy",
@@ -140,6 +145,10 @@ class BucketClient:
         errors: Optional[str] = None,
         newline: Optional[str] = None,
     ) -> StreamableType:
+        client_params = {}
+        if hasattr(self, "client_params"):
+            client_params = getattr(self, "client_params")
+
         return smart_open.open(
             self.make_uri(path),
             mode=mode,
@@ -147,6 +156,7 @@ class BucketClient:
             encoding=encoding,
             errors=errors,
             newline=newline,
+            transport_params=client_params,
             # Disable de/compression based on the file extension
             ignore_ext=True,
         )  # type:ignore
@@ -186,7 +196,7 @@ class BucketClient:
         path: "Pathy" = None,
         prefix: Optional[str] = None,
         delimiter: Optional[str] = None,
-    ) -> Generator[BucketEntry[BucketType, BucketBlobType], None, None]:
+    ) -> "PathyScanDir":
         raise NotImplementedError(SUBCLASS_ERROR)
 
     def create_bucket(self, path: "Pathy") -> Bucket:
@@ -356,12 +366,13 @@ class BucketsAccessor(_Accessor):
         # Determine if the path exists according to the current adapter
         return client.exists(path)
 
-    def scandir(self, path: "Pathy") -> Generator[BucketEntry, None, None]:
+    def scandir(self, path: "Pathy") -> "PathyScanDir":
         return self.client(path).scandir(path, prefix=path.prefix)
 
     def listdir(self, path: "Pathy") -> Generator[str, None, None]:
-        for entry in self.scandir(path):
-            yield entry.name
+        with self.scandir(path) as entries:
+            for entry in entries:
+                yield entry.name
 
     def open(
         self,
@@ -475,11 +486,13 @@ class Pathy(Path, PurePathy):
         return of this function to the desired type, e.g.
 
         ```python
-        fluid_path = FluidPath("gs://my_bucket/foo.txt")
+        from pathy import FluidPath, Pathy
+
+        fluid_path: FluidPath = Pathy.fluid("gs://my_bucket/foo.txt")
         # Narrow the type to a specific class
         assert isinstance(fluid_path, Pathy), "must be Pathy"
         # Use a member specific to that class
-        print(fluid_path.prefix)
+        assert fluid_path.prefix == "foo.txt/"
         ```
         """
         from_path: FluidPath = Pathy(path_candidate)
@@ -493,6 +506,8 @@ class Pathy(Path, PurePathy):
         the appropriate prefix.
 
         ```python
+        from pathy import Pathy
+
         assert str(Pathy.from_bucket("one")) == "gs://one/"
         assert str(Pathy.from_bucket("two")) == "gs://two/"
         ```
@@ -634,7 +649,7 @@ class Pathy(Path, PurePathy):
             raise ValueError("binary mode doesn't take an encoding argument")
 
         # Leftover pathlib internals stuff
-        if self._closed:  # type:ignore
+        if hasattr(self, "_closed") and self._closed:  # type:ignore
             self._raise_closed()  # type:ignore
         return self._accessor.open(
             self,
@@ -658,6 +673,8 @@ class Pathy(Path, PurePathy):
         """Resolve the given path to remove any relative path specifiers.
 
         ```python
+        from pathy import Pathy
+
         path = Pathy("gs://my_bucket/folder/../blob")
         assert path.resolve() == Pathy("gs://my_bucket/blob")
         ```
@@ -665,7 +682,7 @@ class Pathy(Path, PurePathy):
         self._absolute_path_validation()
         return self._accessor.resolve(self, strict=strict)
 
-    def rename(self: "Pathy", target: Union[str, PurePath]) -> None:
+    def rename(self: "Pathy", target: Union[str, PurePath]) -> "Pathy":  # type:ignore
         """Rename this path to the given target.
 
         If the target exists and is a file, it will be replaced silently if the user
@@ -675,16 +692,16 @@ class Pathy(Path, PurePathy):
         to match the target prefix."""
         self._absolute_path_validation()
         self_type = type(self)
-        if not isinstance(target, self_type):
-            target = self_type(target)
-        target._absolute_path_validation()  # type:ignore
-        super().rename(target)
+        result = target if isinstance(target, self_type) else self_type(target)
+        result._absolute_path_validation()  # type:ignore
+        super().rename(result)
+        return result
 
-    def replace(self: "Pathy", target: Union[str, PurePath]) -> None:
+    def replace(self: "Pathy", target: Union[str, PurePath]) -> "Pathy":  # type:ignore
         """Renames this path to the given target.
 
         If target points to an existing path, it will be replaced."""
-        self.rename(target)
+        return self.rename(target)
 
     def rmdir(self: "Pathy") -> None:
         """Removes this bucket or blob prefix. It must be empty."""
@@ -807,3 +824,42 @@ class Pathy(Path, PurePathy):
             method=self.symlink_to.__qualname__
         )
         raise NotImplementedError(message)
+
+
+class PathyScanDir(Iterator[Any], ContextManager[Any]):
+    """A scandir implementation that works for all python 3.x versions.
+
+    Python < 3.7 requires that scandir be iterable so it can be converted
+    to a list of results.
+
+    Python >= 3.8 requires that scandir work as a context manager.
+    """
+
+    def __init__(
+        self,
+        client: BucketClient,
+        path: Optional[PurePathy] = None,
+        prefix: Optional[str] = None,
+        delimiter: Optional[str] = None,
+    ) -> None:
+        super().__init__()
+        self._client = client
+        self._path = path
+        self._prefix = prefix
+        self._delimiter = delimiter
+        self._generator = self.scandir()
+
+    def __enter__(self) -> Generator[BucketEntry, None, None]:
+        return self._generator
+
+    def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any) -> None:
+        pass
+
+    def __next__(self) -> Generator[BucketEntry, None, None]:
+        yield from self._generator
+
+    def __iter__(self) -> Generator[BucketEntry, None, None]:
+        yield from self._generator
+
+    def scandir(self) -> Generator[BucketEntry, None, None]:
+        raise NotImplementedError("must be implemented in a subclass")
