@@ -129,6 +129,13 @@ def test_base_repr():
     assert bytes(PurePathy("fake_file.txt")) == b"fake_file.txt"
 
 
+def test_base_scheme_extraction():
+    assert PurePathy("gs://var/tests/fake").scheme == "gs"
+    assert PurePathy("s3://var/tests/fake").scheme == "s3"
+    assert PurePathy("file://var/tests/fake").scheme == "file"
+    assert PurePathy("/var/tests/fake").scheme == ""
+
+
 @pytest.mark.skipif(sys.version_info < (3, 6), reason="requires python3.6 or higher")
 def test_base_fspath():
     assert os.fspath(PurePathy("/var/tests/fake")) == "/var/tests/fake"
@@ -756,6 +763,7 @@ def test_api_raises_with_no_known_bucket_clients_for_a_scheme(temp_folder):
     assert isinstance(accessor.client(path), BucketClientFS)
 
 
+@pytest.mark.skip("requires: https://github.com/explosion/thinc/pull/465")
 def test_api_export_spacy_model(temp_folder):
     """spaCy model loading is one of the things we need to support"""
     use_fs(temp_folder)
